@@ -12,7 +12,7 @@
     unused_qualifications
 )]
 
-/* Standard Library Dependencies */
+// Standard Library Dependencies
 use core::fmt::{self, Debug, Display};
 use std::error::Error;
 
@@ -50,6 +50,39 @@ fn error_sources(mut source: Option<&(dyn Error + 'static)>) -> Option<Vec<Strin
  * TRAIT *
  *********/
 
+// Helper Functions
+
+/// Convenience function that converts the summary into pretty String.
+fn pretty_summary(summary: &str) -> String {
+    [SUMMARY_PREFIX, summary, RESET].concat()
+}
+
+/// Convenience function that converts the reasons into pretty String.
+fn pretty_reasons(reasons: Reasons) -> Option<String> {
+    /* Print list of Reasons (if any) */
+    if let Some(reasons) = reasons {
+        /* Vector to store the intermediate bullet point strings */
+        let mut reason_strings = Vec::with_capacity(reasons.len());
+        for reason in reasons {
+            let bullet_point = [REASON_PREFIX, &reason].concat();
+            reason_strings.push(bullet_point);
+        }
+        /* Join the buller points with a newline, append a RESET ASCII escape code to the end */
+        Some([&reason_strings.join("\n"), RESET].concat())
+    } else {
+        None
+    }
+}
+
+/// Convenience function that converts the help text into pretty String.
+fn pretty_helptext(helptext: Helptext) -> Option<String> {
+    if let Some(helptext) = helptext {
+        Some([HELPTEXT_PREFIX, &helptext, RESET].concat())
+    } else {
+        None
+    }
+}
+
 /// You can implement UFE on your error types pretty print them. The default implementation will print Error: <your error .to_string()> followed by a list of reasons that are any errors returned by .source()
 /// You should only override the summary, reasons and helptext functions. The pretty print versions of these are used by print(), print_and_exit() and contain the formatting. If you wish to change the formatting you should update it with the formatting functions.
 pub trait UFE: Error {
@@ -57,7 +90,7 @@ pub trait UFE: Error {
      * IMPLENT ME *
      **************/
 
-    /// Returns a summary of the error. This will be printed in red, prefixed by "Error: ", at the top of the error message. This is not Optional.
+    /// Returns a summary of the error. This will be printed in red, prefixed by "Error: ", at the top of the error message.
     fn summary(&self) -> String {
         self.to_string()
     }
@@ -73,41 +106,6 @@ pub trait UFE: Error {
         None
     }
 
-    /****************
-     * PRETTY PRINT *
-     ****************/
-
-    /// Convenience function that converts the summary into pretty String. You shouldn't implement this.
-    fn pretty_summary(&self) -> String {
-        [SUMMARY_PREFIX, &self.summary(), RESET].concat()
-    }
-
-    /// Convenience function that converts the reasons into pretty String. You shouldn't implement this.
-    fn pretty_reasons(&self) -> Option<String> {
-        /* Print list of Reasons (if any) */
-        if let Some(reasons) = self.reasons() {
-            /* Vector to store the intermediate bullet point strings */
-            let mut reason_strings = Vec::with_capacity(reasons.len());
-            for reason in reasons {
-                let bullet_point = [REASON_PREFIX, &reason].concat();
-                reason_strings.push(bullet_point);
-            }
-            /* Join the buller points with a newline, append a RESET ASCII escape code to the end */
-            Some([&reason_strings.join("\n"), RESET].concat())
-        } else {
-            None
-        }
-    }
-
-    /// Convenience function that converts the help text into pretty String. You shouldn't implement this.
-    fn pretty_helptext(&self) -> Option<String> {
-        if let Some(helptext) = self.helptext() {
-            Some([HELPTEXT_PREFIX, &helptext, RESET].concat())
-        } else {
-            None
-        }
-    }
-
     /**********
      * USE ME *
      **********/
@@ -123,15 +121,15 @@ pub trait UFE: Error {
     /// ```
     fn print(&self) {
         /* Print Summary */
-        eprintln!("{}", self.pretty_summary());
+        eprintln!("{}", pretty_summary(&self.summary()));
 
         /* Print list of Reasons (if any) */
-        if let Some(reasons) = self.pretty_reasons() {
+        if let Some(reasons) = pretty_reasons(self.reasons()) {
             eprintln!("{}", reasons);
         }
 
         /* Print help text (if any) */
-        if let Some(helptext) = self.pretty_helptext() {
+        if let Some(helptext) = pretty_helptext(self.helptext()) {
             eprintln!("{}", helptext);
         }
     }
@@ -149,6 +147,45 @@ pub trait UFE: Error {
         self.print();
         std::process::exit(1)
     }
+
+    /// Consumes the UFE and returns a UserFacingError. Useful if you want access to additional functions to edit the error message before exiting the program.
+    /// # Example
+    /// ```
+    /// use user_error::{UserFacingError, UFE};
+    /// use std::fmt::{self, Display};
+    /// use std::error::Error;
+    ///
+    /// #[derive(Debug)]
+    /// struct MyError {}
+    ///
+    /// impl Display for MyError {
+    ///     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+    ///         write!(f, "MyError")
+    ///     }
+    /// }
+    ///
+    /// impl Error for MyError {
+    ///     fn source(&self) -> Option<&(dyn Error + 'static)> { None }
+    /// }
+    ///
+    /// impl UFE for MyError {}
+    ///
+    /// fn main() {
+    ///     let me = MyError {};
+    ///     me.print();
+    ///     me.into_ufe()
+    ///         .help("Added helptext")
+    ///         .print();
+    /// }
+    /// ```
+    fn into_ufe(&self) -> UserFacingError {
+        UserFacingError {
+            summary: self.summary(),
+            reasons: self.reasons(),
+            helptext: self.helptext(),
+            source: None,
+        }
+    }
 }
 
 /**********
@@ -173,26 +210,12 @@ pub struct UserFacingError {
  * IMPLEMENTATION *
  ******************/
 
-// Implement our own trait for our example struct
-// Cloning is not super efficient but this should be the last thing a program does, and it will only do it once so... ¯\_(ツ)_/¯
-impl UFE for UserFacingError {
-    fn summary(&self) -> Summary {
-        self.summary.clone()
-    }
-    fn reasons(&self) -> Reasons {
-        self.reasons.clone()
-    }
-    fn helptext(&self) -> Helptext {
-        self.helptext.clone()
-    }
-}
-
 // Implement Display so our struct also implements std::error::Error
 impl Display for UserFacingError {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        let summary = self.pretty_summary();
-        let reasons = self.pretty_reasons();
-        let helptext = self.pretty_helptext();
+        let summary = pretty_summary(&self.summary());
+        let reasons = pretty_reasons(self.reasons());
+        let helptext = pretty_helptext(self.helptext());
 
         /* Love this - thanks Rust! */
         match (summary, reasons, helptext) {
@@ -216,6 +239,20 @@ impl Error for UserFacingError {
     }
 }
 
+// Implement our own trait for our example struct
+// Cloning is not super efficient but this should be the last thing a program does, and it will only do it once so... ¯\_(ツ)_/¯
+impl UFE for UserFacingError {
+    fn summary(&self) -> Summary {
+        self.summary.clone()
+    }
+    fn reasons(&self) -> Reasons {
+        self.reasons.clone()
+    }
+    fn helptext(&self) -> Helptext {
+        self.helptext.clone()
+    }
+}
+
 // Helper function to keep things DRY
 fn get_ufe_struct_members(error: &(dyn Error)) -> (Summary, Reasons) {
     /* Error Display format is the summary */
@@ -225,6 +262,11 @@ fn get_ufe_struct_members(error: &(dyn Error)) -> (Summary, Reasons) {
     (summary, reasons)
 }
 
+//
+// Instead of converting from std Error, convert from types that implement UFE :)
+// Don't have my type implement std Error
+//
+
 /// Allows you to create UserFacingErrors From std::io::Error for convenience
 /// You should really just implement UFE for your error type, but if you wanted to convert before quitting so you could add helptext of something you can use this.
 impl From<std::io::Error> for UserFacingError {
@@ -287,7 +329,6 @@ impl<T: Debug> From<Result<T, Box<dyn Error>>> for UserFacingError {
     }
 }
 
-// Implement convenience functions to modify the UserFacingError struct
 impl UserFacingError {
     /// This is how users create a new User Facing Error. The value passed to new() will be used as an error summary. Error summaries are displayed first, prefixed by 'Error: '.
     /// # Example
@@ -327,7 +368,7 @@ impl UserFacingError {
         let old_summary = self.summary();
         match self.reasons.as_mut() {
             Some(reasons) => reasons.insert(0, old_summary),
-            None => self.reasons = Some(vec![old_summary.into()]),
+            None => self.reasons = Some(vec![old_summary]),
         }
 
         // Update the summary
@@ -353,6 +394,8 @@ impl UserFacingError {
         self
     }
 
+    // Return ref to previous?
+
     /// Clears all reasons from a UserFacingError.
     /// # Example
     /// ```
@@ -579,4 +622,41 @@ mod tests {
             side: SuperErrorSideKick,
         }))
     }
+
+    // Custom Error Type
+    #[derive(Debug)]
+    struct MyError {
+        mssg: String,
+        src: Option<Box<dyn Error>>,
+    }
+
+    impl Display for MyError {
+        fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+            write!(f, "{}", self.mssg.to_string())
+        }
+    }
+
+    impl Error for MyError {
+        fn source(&self) -> Option<&(dyn Error + 'static)> {
+            self.src.as_deref()
+        }
+    }
+
+    impl UFE for MyError {}
+
+    #[test]
+    fn custom_error_implements_ufe() {
+        let me = MyError {
+            mssg: "Program Failed".into(),
+            src: Some(Box::new(MyError {
+                mssg: "Reason 1".into(),
+                src: Some(Box::new(MyError {
+                    mssg: "Reason 2".into(),
+                    src: None,
+                })),
+            })),
+        };
+        me.print();
+        me.into_ufe().help("Helptext Added").print();
+    }
 }
